<?php
/* VR pay eCommerce easyCredit model
 *
 * @version 2.3.03
 * @date 2018-01-03
 *
 */
include_once(dirname(__FILE__) . '/../../vrpayecommerce/vrpayecommerce.php');

class ModelExtensionPaymentVrpayecommerceEasycredit extends ModelVrpayecommerceVrpayecommerce {
	protected $code = 'vrpayecommerce_easycredit';
	protected $title = 'FRONTEND_PM_EASYCREDIT';
	protected $logo = 'easycredit.png';

	/**
	 * Get the Method
	 * this funtion is the OpenCart funtion
	 *
	 * @param string $address
	 * @param int $total
	 * @return  array
	 */
	public function getMethod($address, $total)
	{

		$this->language->load('extension/payment/vrpayecommerce');
		$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "zone_to_geo_zone WHERE geo_zone_id = '" . (int)$this->config->get('vrpayecommerce_cc_geo_zone_id') . "' AND country_id = '" . (int)$address['country_id'] . "' AND (zone_id = '" . (int)$address['zone_id'] . "' OR zone_id = '0')");

		if ($this->config->get($this->code . '_total') > 0 && $this->config->get($this->code . '_total') > $total) {
			$status = false;
		} elseif (!$this->config->get($this->code . '_geo_zone_id')) {
			$status = true;
		} elseif ($query->num_rows) {
			$status = true;
		} else {
			$status = false;
		}

		$easycredit_error_messages = $this->getEasycreditErrorMessage($address, $total);

		$method_data = array();
		if ($status) {
			$logo = $this->getLogo();
			if (!empty($easycredit_error_messages)) {
				$logo .= $this->getDisabledMessage($easycredit_error_messages);
			}
			$method_data = array(
				'code'       	=> $this->code,
				'title'      	=> $this->language->get($this->title),
				'logo'			=> $logo,
				'terms'		 	=> '',
				'sort_order' 	=> $this->config->get($this->code . '_sort_order')
			);
		}
		return $method_data;
	}

	/**
	 * Validate all required data when using easy credit payment method
	 *
	 * @param string $address
	 * @param int $total
	 * @return array
	 */
	public function getEasycreditErrorMessage($address, $total)
	{

		$virtual_item = $this->isVirtualItem();
		$currency_query = $this->db->query("SELECT * FROM " . DB_PREFIX . "currency WHERE code = '" . $this->session->data['currency'] . "'");
		$euro_currency_value = $currency_query->row['value'];
		$total_amount = (float)$total * $euro_currency_value;
		$amount = number_format($total_amount, 2, '.', '');
		$error_messages = array();

		if (!$this->isAmountAllowed($amount)) {
			$error_messages[] = 'ERROR_MESSAGE_EASYCREDIT_AMOUNT_NOTALLOWED';
		}
		if (!$this->isBillingEqualShipping($address) || $this->isVirtualItem()) {
			$error_messages[] = 'ERROR_EASYCREDIT_BILLING_NOTEQUAL_SHIPPING';
		}

		if (!$this->isDateOfBirthValid()) {
			$error_messages[] = 'ERROR_EASYCREDIT_PARAMETER_DOB';
		}
		if (!$this->isDateOfBirthLowerThanToday()) {
			$error_messages[] = 'ERROR_EASYCREDIT_FUTURE_DOB';
		}

		if(!$this->getCustomerSex()) {
			$error_messages[] = 'ERROR_MESSAGE_EASYCREDIT_PARAMETER_GENDER';
		}

		return $error_messages;

	}

	/**
	 * is valid date of birth
	 *
	 * @return boolean
	 */
	public function isDateOfBirthValid()
	{
		$year = 0;
		$month = 0;
		$day = 0;

		$customer_dob = $this->getCustomerDob();
		if ($customer_dob) {
		    $customer_date_of_birth = explode("-", $customer_dob);

		    if (isset($customer_date_of_birth[0])) {
			    $year = (int)$customer_date_of_birth[0];
			}
		    if (isset($customer_date_of_birth[1])) {
			    $month = (int)$customer_date_of_birth[1];
			}
		    if (isset($customer_date_of_birth[2])) {
			    $day = (int)$customer_date_of_birth[2];
			}
		}

	    if ($year < 1900) {
	        return false;
	    }

	    if ($month < 1 || $month > 12) {
	        return false;
	    }

	    if ($day < 1 || $day > 31) {
	        return false;
	    }

	    $valid = checkdate($month, $day, $year);

	    if (!$valid) {
	        return false;
	    }
	    return true;
	}


	/**
	 * is date of birth lower than today
	 *
	 * @return boolean
	 */
	public function isDateOfBirthLowerThanToday()
	{
		$customer_date_of_birth = strtotime($this->getCustomerDob());
		$today = strtotime(date('Y-m-d'));

		if ($customer_date_of_birth < $today) {
			return true;
		}
		return false;
	}

	/**
	 * is total amount allowed
	 *
	 * @param string $total
	 * @return boolean
	 */
	protected function isAmountAllowed($total) {
		if ((int)$total >= 200 && (int)$total <= 5000 && $this->session->data['currency'] == 'EUR') {
			return true;
		}
		return false;
	}

	/**
	 * Validate if billing address equal shipping or not
	 *
	 * @param string $address
	 * @return boolean
	 */
	protected function isBillingEqualShipping($address) {
		$shipping_address = $this->isShippingAddressExist();
		if (empty($shipping_address) && !empty($address)) {
			return true;
		}
		if ($address == $this->customer->session->data['shipping_address']) {
			return true;
		}
	    return false;
	}

	/**
	 * Validate if item is virtual or not
	 *
	 * @return boolean
	 */
	protected function isVirtualItem() {
		$data['products'] = array();
		$products = $this->cart->getProducts();
		$count_item = 0;
		$count_item_virtual = 0;
		$count_item_non_virtual = 0;
		$virtual_item = false;

		foreach ($products as $product) {
			$query = $this->db->query("SELECT shipping FROM `" . DB_PREFIX . "product` WHERE product_id=" . $product['product_id']);
			if ($query->row['shipping'] == 0){
				$virtual_item = true;
				$count_item_virtual++;
			} else {
				$count_item_non_virtual++;
			}
			$count_item++;
		}

		if ($virtual_item == true && $count_item == 1){
			return true;
		} else if ($virtual_item == true && $count_item_virtual > 1 && $count_item_non_virtual == 0){
			return true;
		}
		return false;
	}

	/**
	 * Check customer shipping address session
	 *
	 * @return string
	 */
	protected function isShippingAddressExist() {
		if (!empty($this->session->data['shipping_address'])) {
			return true;
		}
		return false;
	}

	/**
	 * Get the date of birth of a customer
	 *
	 * @return string | boolean
	 */
	protected function getCustomerDob() {

		if ($this->customer->isLogged()) {
			$customers = $this->getCustomerData();
			$custom_field = json_decode($customers[0]['custom_field'], true);
			$id = $this->config->get('vrpayecommerce_ageneral_dob_customfield');
			if (isset($custom_field[$id])) {
				return $custom_field[$id];
			}
		} else {
			if (isset($this->session->data['guest']['custom_field'][2])) {
				return $this->session->data['guest']['custom_field'][2];
			}
		}
		return false;
	}

	/**
	 * Get a customer sex
	 *
	 * @param array $custom_field
	 * @return  string | boolean
	 */
	function getCustomerSex() {

		if ($this->customer->isLogged()) {
			$customers = $this->getCustomerData();
			$custom_field = json_decode($customers[0]['custom_field'], true);
		} else {
			$custom_field = $this->session->data['guest']['custom_field'];
		}

		$id = $this->config->get('vrpayecommerce_ageneral_gender_customfield');
		$male_value = $this->config->get('vrpayecommerce_ageneral_gender_malevalue');
		$female_value = $this->config->get('vrpayecommerce_ageneral_gender_femalevalue');
		if (isset($custom_field[$id])) {
			switch ($custom_field[$id]) {
				case $male_value:
					return 'M';
					break;
				case $female_value:
					return 'F';
					break;
			}
		}
		return false;
	}

	protected function getDisabledMessage($error_messages) {
		$this->language->load('extension/payment/vrpayecommerce');
		$disabled_message = '';
		foreach ($error_messages as $value) {
			$disabled_message .= '<span style="display:block; padding:10px; background:#efefef;">* ' . $this->language->get($value) . '</span>';
		}
		$disabled_message .= '<script>$("input[value=\'vrpayecommerce_easycredit\']").attr("disabled", "disabled").removeAttr("checked");</script>';

		return $disabled_message;
	}
}
